/***************************************************************************
 *                                                                         *
 *                            LibQNotify                                   *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#ifndef QNOTIFY_POPUP_H
#define QNOTIFY_POPUP_H

/** @defgroup libqnotify LibQNotify
 * @{ */

#include "qnotify_container.h"

class QNotifyProgressWidget;

/** @brief Predefined positons on the desktop */
enum QNOTIFY_POSITION { QNOTIFY_POS_UNDEFINED,
			QNOTIFY_POS_TOP_LEFT,
			QNOTIFY_POS_TOP_MID,
			QNOTIFY_POS_TOP_RIGHT,
			QNOTIFY_POS_MID_LEFT,
			QNOTIFY_POS_MID_MID,
			QNOTIFY_POS_MID_RIGHT,
			QNOTIFY_POS_BOTTOM_LEFT,
			QNOTIFY_POS_BOTTOM_MID,
			QNOTIFY_POS_BOTTOM_RIGHT };
			
/** @brief Class being a passive, unfocused popup on the desktop */
class QNotifyPopup : public QNotifyContainer {
	Q_OBJECT

public:
	/** @brief contructor initializing default values
	 *
	 * It is not required to give any parameters
	 */
	QNotifyPopup(QNotifyPopup *parent = 0, const char *name = 0,
		     WFlags f = WType_TopLevel |
		                WNoAutoErase |
		                WStyle_Customize |
		                WX11BypassWM |
	                        WStyle_StaysOnTop);

	/** @brief destructor */
	~QNotifyPopup();

	/** @brief pops up a predefined popup
	 *
	 * @param text text displayed
	 */
	static void popup(const QString &text);

	/** @brief add a extended text widget to the popup
	 *
	 * Adds a @ref QNotifyLabel to the popup below the previous widget
	 * if there is one. By default, the text will have a shadow, a
	 * font size of 12 and will be aligned centered
	 *
	 * @param text the text to be shown
	 * @param fontsize
	 * @param alignment a Qt alignment flag. Atm, only AlignLeft,
	 *                  AlignHCenter and AlignRight are supported
	 */
	void addTextWidget(const QString &text, int fontsize = 12,
			   int alignment = Qt::AlignHCenter);

	/** @brief adds a progress bar to the popup
	 *
	 * @param height the height of the progress bar
	 * @param color the color of the progress bar
	 */
	void addProgressBar(int height = -1, const QString &color = "#3b508a");

	/** @brief set the progress of the progress bar if any
	 *
	 * @param progress the progress from 0 up to 100
	 * @param text the text to display in the bar
	 */
	void setProgress(int progress, const QString &text = "");

	/** @brief add a custom widget which is derived from QWidget
	 *
	 * Adds a custom widget to the popup below the previous widget if
	 * there is one.
	 *
	 * @param widget a custom widget which has QWidget as its parent
	 * @param alignment a Qt alignment flag. Atm, only AlignLeft,
	 *                  AlignHCenter and AlignRight are supported
	 */
	void addWidget(QWidget *widget, int alignment = Qt::AlignHCenter);

	/** @brief returns the time in milliseconds when the popup is
	 * finished */
	float timeout();

	/** @brief set the number of times the popup is shown and hidden
	 *
	 * see @ref setShowInterval and @ref setHideInterval
	 *
	 * @param times_shown number of times the popup will be shown
	 */
	void setTimesShown(int times_shown);

	/** @brief time the popup will be visible untils it is hidden
	 *
	 * @param show_time time in milliseconds
	 */
	void setShowInterval(int show_time);

	/** @brief time the popup will be hidden untils it is shown
	 *
	 * @param hide_time time in milliseconds
	 */
	void setHideInterval(int hide_time);

	/** @brief the position on the desktop
	 *
	 * one can give a default position for the popup
	 *
	 * @param pos @ref QNOTIFY_POSITION
	 */
	void setPosition(QNOTIFY_POSITION pos);

	/** @brief set the popup transparent
	 *
	 * for custom widgets, this may result in only showing the text
	 */
	void setTransparent();

	/** @brief the margin between all widgets and the outer frame
	 *
	 * @param margin the margin
	 */
	void setMargin(int margin);

	/** @brief disables the drawing of the shadow of the text */
	void disableShadow();

	/** @brief set the rounding factor of the edges
	 *
	 * @param factor the factor from 0 up to something like 10000
	 *
	 */
	void setRounding(int factor);

	/** @brief let the popup slide over the desktop
	 *
	 * specify coordinates the popup will move to while shown. All
	 * three lists must have the same number of elements
	 *
	 * @param x_positions list of integers representing the x-axis
	 * @param y_positions list of integers representing the y-axis
	 * @param intervals list of integers each specifying a time in
	 *                  milliseconds until the popup moves on to the
	 *                  next position
	 *
	 * @return true if given lists are valid, false otherwise
	 */
	bool setSliding(QValueList< int > x_positions,
			QValueList< int > y_positions,
			QValueList< int > intervals);

	/** @brief set the text of a custom textwidget
	 *
	 * don't forget to call @ref render afterwards
	 *
	 * @param text the text to set
	 * @param widget_num the text widget number starting from 0
	 */
	void setText(const QString &text, int widget_num = 0);

	/** @brief set the foreground color, usually the text displayed
	 *
	 * @param color a Qt color
	 */
	void setTextColor(const QString &color);

	/** @brief set the background color
	 *
	 * @param color a Qt color
	 */
	void setBackgroundColor(const QString &color);

	/** @brief set the popup to be fullscreen
	 *
	 * This sometimes makes problems when using custom widgets
	 */
	void setFullscreen();

	/** @brief renders the popup
	 *
	 * this functions renders the popup, calculating its width and
	 * height, setting up transparency, drawing the background and the
	 * text, etc.
	 *
	 * if -1 is given to the width or the height (which should
	 * preferably be used), the QNotifyPopup engine cares about sizes
	 * and that the whole popup fits on the screen
	 *
	 * @param width the width of the popup
	 * @param height the height of the popup
	 */
	void render(int width = -1, int height = -1);

public slots:

	/** @brief shows the widget and initialzes show and hidetimers */
	void popup();

signals:

	/** @brief emitted when the popup is finished */
        void popupFinished();

private:
	/** @brief sets the position on the desktop
	 *
	 * @param pos @ref QNOTIFY_POSITION predefined position
	 *
	 */
	void setDefaultPosition(QNOTIFY_POSITION pos);

	/** @brief handles a mouse click
	 *
	 * in this case @ref popupFinished is emitted
	 *
	 * @param event the mouse event that occured
	 */
	void mousePressEvent(QMouseEvent * event);

	/** @brief init the sliding engine */
	void initSliding();

	int getHeight();

	/** @brief lists containing the sliding information */
	QValueList < int >_x_positions, _y_positions, _intervals;

	/** @brief the time the popup is shown until hidden */
	int _show_time;

	/** @brief the time the popup is hidden until shown */
	int _hide_time;

	/** @brief number of times the popup is shown */
	int _times_shown;

	/** @brief whether this is a transparent popup */
	bool _transparent;

	/** @brief the position @ref QNOTIFY_POSITION of the popup */
	QNOTIFY_POSITION _default_position;

	/** @brief the outer margin between the frame and its content */
	int _margin;

	/** @brief whether the popup's text draws a shadow */
	bool _shadow;

	/** @brief list of widgets the popup contains */
	QValueList< QWidget* > _widgets;
	
	/** @brief list of alignment flags for @ref _widgets */
	QValueList< int > _align;

	/** @brief the rounding factor of the popup/edges */
	int _rounding_factor;

	/** @brief timer running when the popup is shown until it is
	 * hidden */
	QTimer *showtimer;

	/** @brief timer running when the popup is hidden until it is
	 * shown */
	QTimer *hidetimer;

	/** @brief timer that slides the popup and calls @ref slide */
	QTimer *slideTimer;

	QNotifyProgressWidget *_progress_bar;

private slots:
	/** @brief hides the popup until the showtimer exceeds */
	void hidePopup();

        /** @brief moves to the next position if sliding is active */
	void slide();
};

/** @} */ // end of libqnotify group

#endif // QNOTIFY_POPUP_H
